// hifipix.js

function initializeHifiPix() {
  // Sort viewport breakpoints in numeric order ascending
  window.hifipix_settings.behavior.breakpoints = 
    window.hifipix_settings.behavior.breakpoints.sort( function(a, b){return a - b} );

  jQuery('.wpqd').each( function() {
    jQuery(this).hifiLoad();
   }
  );
}

jQuery.widget( 'wpqd.hifiLoad', {
  image: null,
  loadImage: null,
  enabledImageExtensions: [],
  normalImageExtensions: [],
  events: [],

  _create: function () {
    var that = this;

    
    // List of image types that are loaded without
    // the ProgressivePNG codec
    jQuery.each(window.hifipix_settings.general.file_types, function(type, enabled) {
      if ( ! enabled ) {
        return true;
      }

      that.enabledImageExtensions.push( type );

      // TODO: Uncomment this when progressive PNG functionality works
      // if ( type === 'png' ) {
      //   return true;
      // }

      that.normalImageExtensions.push( type );

      // Also include "jpeg" as a valid extension type
      if ( type === 'jpg' ) {
        that.normalImageExtensions.push( 'jpeg' );
        that.enabledImageExtensions.push( 'jpeg' );
      }
    })

    var jqueryElement = this.element;

    var imagesetAttr = jqueryElement.attr('wpqd-imageset');
    if (imagesetAttr !== undefined) {
      // Note: hifi-imageset JSON uses singlequotes for ability
      // to be contained within html attributes. Replace with
      // double quotes for parsing.
      imagesetAttr  = imagesetAttr.replace(/'/g, "\"");

      var hifiImageset = JSON.parse(imagesetAttr);
      var filename = hifiImageset.wpqd_root_filename + '.' + hifiImageset.wpqd_file_extension;

      hifiCacheManager.init();

      // Build image object with metadata about image
      // including jQuery element for that image
      // to be used by loading functions
      this.image = {
        id: jqueryElement.attr('wpqd-id'),
        jqueryElement: jqueryElement,
        filename: filename,
        type: hifiImageset.wpqd_file_extension,
        imageset: hifiImageset,
        aspectRatio: jqueryElement.attr('wpqd-ar'),
        origin: this.getImageOrigin( jqueryElement.attr('orig_src') ),
        maxSize: parseInt( jqueryElement.attr('wpqd-max') )
      }

      hifiLoadManager.initProperties( this.image.id );

      // Skip disabled image extension types
      if ( ! this.enabledImageExtensions.includes(this.image.type) ) {
        return;
      }

      if (this.image.jqueryElement.attr('height') === undefined || this.image.jqueryElement.attr('height') === 0) {
        // Use image "width" attribute to set initial image size
        this.image.jqueryElement.height( this.image.jqueryElement.attr('width') * (1/this.image.aspectRatio) );
      } else {
        this.image.jqueryElement.height( this.image.jqueryElement.width() * (1/this.image.aspectRatio) );
      }

      if ( window.hifipix_settings.behavior.progress == '1' ) {
        this.initProgressIndicator();
      }

      
      // No-Op image function for debugging
      this.loadImage = function() {
        console.log('loading nothing');
      }

            // free
      this.loadImage = this.loadNormalImage;
      
      // Set various load event handlers
      if ( window.hifipix_settings.behavior.deferred_load == '1' ) {
        if ( window.hifipix_settings.behavior.trigger == 'hover' ) {
          this.setHoverEventHandler();
        } else {
          this.setScrollEventHandler();
        }
      }
      
      this.setDocumentReadyEventHandler();

      if ( window.hifipix_settings.behavior.resize == '1' ) {
        this.events.push('resize');
      }

      
          }
  },

  getImageOrigin: function ( src ) {
    if ( typeof src === 'undefined' ) {
      return window.location.origin;
    }

    if ( src.match(/^(https?:)?\/\//i) === null ) {
      var srcUrl = new URL( src, window.location.origin );
    } else {
      var srcUrl = new URL( src );
    }    
    return srcUrl.origin;
  },

  // Checks if image is already loaded or if it's inside our threshold
  shouldLoadImage: function ( size, scrollY ) {
    // Image is already loaded
    if ( hifiLoadManager.isLoaded(this.image.id, size) ) {
      return false;
    }

    // "Deferred Load" setting is disabled
    if ( ! window.hifipix_settings.behavior.deferred_load ) {
      return true;
    }

    // Image is outside visible threshold
    if ( ! this.isImageVisible(scrollY) ) {
      return false;
    }

    hifiLoadManager.setImageToLoad( this.image.id );
    return true;
  },

  // Checks if image is within "visible" threshold per scroll_sensitivity
  // setting in hifipix.settings.js
  isImageVisible: function ( scrollY ) {
    var elTop               = this.image.jqueryElement.offset().top,
        windowThreshold     = window.innerHeight * (1 + parseFloat(window.hifipix_settings.behavior.scroll_sensitivity)),
        offset              = windowThreshold + scrollY;

    if (offset > elTop) {
      return true;
    }

    return false;
  },

  // Wrapper function for loadImage that checks whether the image
  // should be loaded and sets status in "hifiLoadManager"
  loadImageWrapper: function ( scrollY, size ) {
    if (typeof size === 'undefined') {
      size = this.getImageBreakpoint();
    }

    // Don't load an image size larger than the max breakpoint
    if ( size > this.image.maxSize ) {
      size = this.image.maxSize;
    }

    if ( ! this.shouldLoadImage(size, scrollY) ) {
      return false;
    }

    this.loadImage(size);
    hifiLoadManager.setIsLoaded(this.image.id, size, true);

    this.image.jqueryElement.css('height', '');
    return true;
  },

  // Retrieves the correct image breakpoint according to the parent element's width
  getImageBreakpoint: function () {
    var parentWidth = this.image.jqueryElement.innerWidth(),
        output      = window.hifipix_settings.behavior.breakpoints[window.hifipix_settings.behavior.breakpoints.length - 1];

    jQuery.each(window.hifipix_settings.behavior.breakpoints, function(i, breakpoint) {
        if (parentWidth <= breakpoint) {
          output = breakpoint;
          return false;
        }
      });

    return output;
  },

  setDocumentReadyEventHandler: function () {
    var that = this;

    function onReady() {
      that.loadImageWrapper(window.scrollY);
    }
    jQuery(document).ready(onReady);
  },

  // Load images when cursor hovers over them
  setHoverEventHandler: function () {
    var that = this;

    function onHover() {
        that.loadImageWrapper(window.scrollY);
    }

    this.image.jqueryElement.on( 'mouseover', onHover );
  },

  // Load images while scrolling through page
  setScrollEventHandler: function () {
    var that            = this,
        lastScrollY     = 0,
        ticking         = false;

     // Keeps track of the last scroll value
    function onScroll() {
        lastScrollY = window.scrollY;
        requestTick();
    }

    // Prevents multiple calls to rAF (otherwise it's a major performance issue)
    function requestTick() {
        if (!ticking) {
            requestAnimationFrame(update);
            ticking = true;
        }
    }

    // rAF callback
    function update() {
        that.loadImageWrapper(lastScrollY);
        ticking = false;
    }

    jQuery(window).on('scroll', onScroll);
  },

  
   // Loading Functions

  // Progressively load images in an imageset
  // by recursively calling self and incrementing
  // array index of imageset
  loadNormalImage: function ( breakpoint, indexToLoad ) {
    var that = this;
    if (indexToLoad === undefined) {
      indexToLoad = 0;
    }

    if (breakpoint === undefined || breakpoint === 'default') {
      breakpoint = '1200';
    }

    var is = this.image.imageset;
    if (is !== undefined) {
      if (indexToLoad < window.hifipix_settings.behavior.ratios.length) {
        var newImagePath = "/wp-content/uploads/wpqd/" + is.wpqd_root_filename + '_' + breakpoint + '_' + window.hifipix_settings.behavior.ratios[indexToLoad] + '.' + is.wpqd_file_extension;
        var largestImagePath = "/wp-content/uploads/wpqd/" + is.wpqd_root_filename + '_' + breakpoint + '_' + window.hifipix_settings.behavior.ratios[window.hifipix_settings.behavior.ratios.length - 1] + '.' + is.wpqd_file_extension;
        // are we loading smaller images, between the breakpoints?

        // Toggle CSS classes for transition / fade effects
        this.applyTransitions( indexToLoad + 1, false);

        // Set height based on hf-ar
        if (this.image.jqueryElement.width() > 0) {
          this.responsiveResize();
        }

        // Check if the largest image of the set we are about to load
        // is cached in localStorage via hifiCacheManager
        // If image is cached, skip progressive loading process and go 
        // straight to the src of largest image
        if (hifiCacheManager.isCached(largestImagePath)) {
          this.applyTransitions( window.hifipix_settings.behavior.ratios.length, true);
          this.image.jqueryElement[0].src = this.image.origin + largestImagePath;
        } else {
          jQuery.get(newImagePath).then(function() {
            that.image.jqueryElement[0].src = that.image.origin + newImagePath;

            jQuery(that.image.jqueryElement[0]).load(function(){
              // Put src in cache
              hifiCacheManager.add(newImagePath);

              indexToLoad = indexToLoad + 1;

              that.loadNormalImage(breakpoint, indexToLoad);
            });
          });
        }
      }
    }
  },

  
  responsiveResize: function () {
    var that = this;
    this.image.jqueryElement
      .height( this.image.jqueryElement.width() * (1/this.image.aspectRatio) )
      .promise()
      .done( function () {
        that.checkImageHeight();
    });
  },

  // Check if CSS height matches actual image height
  checkImageHeight: function() {
    var that = this;

    if ( parseInt(that.image.jqueryElement[0].getBoundingClientRect().height) != parseInt(this.image.jqueryElement[0].style.height.slice(0, -2)) ) {
      setTimeout( function() {
        that.checkImageHeight()
      }, 100);
    } else {
      hifiLoadManager.unsetImageToLoad( this.image.id );
    }
  },

  // Toggle CSS classes which provide opacity & contrast effects
  // @see assets/css/transitions.css
  applyTransitions: function ( index, cached ) {
    var classIndex = ( index > 4 ) ? 4 : index,
        classPrefix = ( this.normalImageExtensions.includes(this.image.type) ) ? 'j' : 'p';

      this.applyProgressTransitions( classIndex, cached );

    },

  // Wrap images in container div, then append loader icon HTML
  initProgressIndicator: function () {
    this.image.jqueryElement.wrap( '<div class="img-loader-wrap"></div>' );
    
    var leftOffset = this.image.jqueryElement[0].offsetLeft;
    this.image.jqueryElement.parent().append( 
      '<div class="loader-wrap draw loader-paused" style="left:' + leftOffset + 'px;" >\
        <div class="square-1 loader-paused"></div>\
        <div class="square-2 square loader-paused"></div>\
        <div class="square-3 square loader-paused"></div>\
        <div class="square-4 square loader-paused"></div>\
        <div class="square-5 square loader-paused"></div>\
        <div class="border-square loader-paused"></div>\
      </div>'
    );
  },

  // Progressively trigger loader icon keyframe animations
  applyProgressTransitions: function ( index, cached ) {
    if ( ! window.hifipix_settings.behavior.progress ) {
      return;
    }

    var $loaderWrap = this.image.jqueryElement.parent(),
        selector = '';
    
    switch (index) {
      case 1:
        selector = '.square-1, .border-square';
        break;
      case 2:
        selector = '.square-2';
        break;
      case 3:
        selector = '.square-3, .square-4';
        break;
      default:
        selector = '.square-5, .loader-wrap';
    }

    // If image cached in local storage, there are only 2 loading steps
    if ( cached ) {
      selector += ', .square-2, .square-3, .square-4';
    }
    
    jQuery( selector, $loaderWrap ).removeClass( 'loader-paused' ).addClass( 'loader-play' );

    // Complete animation & then hide the icon
    if ( index >= 4 ) {
      jQuery( '.border-square', $loaderWrap ).addClass( 'loader-play-after' );
      setTimeout( function () {
        jQuery( '.loader-wrap', $loaderWrap ).fadeOut(1800);
      }, 700);
    }
  }
}); // end hifiLoad widget

var hifiLoadManager = {
  imagesToLoad: [],
  _imagesToLoad: [],
  secondPass: false,
  loadedImages: {},

  initProperties: function (id) {
    var scope = this;

    this.loadedImages[id] = {
      'loaded': {}
    };

    jQuery.each(window.hifipix_settings.behavior.breakpoints, function(i, breakpoint) {
        scope.loadedImages[id].loaded[breakpoint] = false;
    });
  },

  setIsLoaded: function (id, size, isLoaded) {
    this.loadedImages[id].loaded[size] = isLoaded;
    return this;
  },

  isLoaded: function (id, size) {
    return this.loadedImages[id].loaded[size];
  },

  setImageToLoad: function (id) {
    this.imagesToLoad.push( id );
    this._imagesToLoad.push( id );
    return this.imagesToLoad;
  },

  unsetImageToLoad: function (id) {
    var index = this.imagesToLoad.indexOf(id);
    if (index !== -1) {
      this.imagesToLoad.splice(index, 1);
    }

    if ( this.imagesToLoad.length === 0 ) {
      this.initSecondLoad();
    }
    return this.imagesToLoad;
  },

  initSecondLoad: function () {
    var that = this;

    if ( this.secondPass === true ) {
      return;
    }

    if ( this._imagesToLoad.length === 0 ) {
      return;
    }

    jQuery('.wpqd').each( function() {
      var id = jQuery(this).attr( 'wpqd-id' );

      if ( ! that._imagesToLoad.includes( id ) ) {
        jQuery(this).hifiLoad( 'destroy' ).hifiLoad();
      }
    });

    this.secondPass = true;
  }
};

var hifiCacheManager = {
  ls: [],

  init: function() {
    if(localStorage.hfp_cf && localStorage.hfp_cf !== undefined && localStorage.hfp_cf !== "") {
      this.ls = JSON.parse(localStorage.hfp_cf);
    } else {
      localStorage.hfp_cf = JSON.stringify(this.ls);
    }
  },

  add: function(src) {
    this.ls.push(src);
    this.save();
  },

  save: function() {
    localStorage.hfp_cf = JSON.stringify(this.ls);
  },

  isCached: function(src) {
    return localStorage.hfp_cf.includes(src);
  }
}
