window.hifipixBatchImporter = {
    needsRegenerate: false
};

jQuery(document).ready( function($) {
    // Rerun compatibility check at top of admin pages
    $('body').on('click', '#wpqd_compatibility_check', function (e) {
        e.preventDefault();

        $.ajax({
            url: ajaxurl,
            data: {
                'action': 'wpqd_compatibility_check'
            },
            success: function(data) {
                // Refresh page to update error messages
                location.reload(true);
            }
        });
    });

    // Update W3TC CDN "Custom File List" Setting
    $('body').on('click', '#wpqd_custom_file_cdn', function (e) {
        e.preventDefault();

        var $container = $(this).parent().parent(),
            $responseArea = $('#wpqd_custom_file_cdn_feedback', $container);

        $container.append( '<div class="wpqd-spinner"></div>' ); // AJAX loader
        $responseArea.empty(); // Clear existing feedback

        $.ajax({
            url: ajaxurl,
            data: {
                'action': 'wpqd_custom_file_cdn_update'
            },
            success: function(data) {
                var response = JSON.parse(data);

                if ( response === true ) {
                    $container.addClass('notice-success').removeClass('notice-warning');
                    $responseArea.append( '<p><strong>Your W3TC settings were updated successfully!</strong></p>' );    
                } else {
                    $container.addClass('notice-error').removeClass('notice-warning');
                    $responseArea.append( '<p><strong>There was a problem updating your W3TC settings. If the issue persists, please update your settings manually.</strong></p>' );
                }
                $('.wpqd-spinner', $container).remove(); // Remove AJAX loader
            }
        });
    });

    // Run compatibility check in Tools > Compatibility Check (settings page)
    $('body').on('click', '#wpqd_data-compatibility_check .button', function (e) {
        e.preventDefault();

        var $fieldset = $('#wpqd_data-compatibility_check'),
            $responseArea = $('<div id="compatibility_check-response"></div>');

        $fieldset.append( '<div class="wpqd-spinner"></div>' ); // AJAX loader

        $.ajax({
            url: ajaxurl,
            data: {
                'action': 'wpqd_compatibility_check_ajax'
            },
            success: function(data) {
                $('#compatibility_check-response').remove(); // Clear existing feedback
                $fieldset.append($responseArea);
                $('.wpqd-spinner', $fieldset).remove(); // Remove AJAX loader

                var $responseSelector = $('#compatibility_check-response');

                if ( data === '1') { // Compatibility check passed
                    $responseSelector
                        .addClass('notice notice-success')
                        .html( '<p><strong>Passed compatibility check</strong></p>' );
                    return;
                }

                var errors = JSON.parse(data);
                $responseSelector
                    .addClass('notice notice-error')
                    .html( '<p><strong>Compatibility check failed</strong></p>' )
                    .append( '<ul class="errors"></ul>' );

                $.each(errors, function(k, v) {
                    $('.errors', $responseSelector).append('<li>' + v + '</li>');
                });
            }
        });
    });

    // Dismiss compatibility check errors on "X" icon click
    $('body').on('click', '#wpqd-warnings .notice-dismiss', function (e) {
        $.ajax({
            url: ajaxurl,
            data: {
                'action': 'wpqd_dismiss_admin_error'
            }
        });
    });

    // Dismiss filesystem errors on "X" icon click
    $('body').on('click', '#wpqd-warnings-filesystem .notice-dismiss', function (e) {
        $.ajax({
            url: ajaxurl,
            data: {
                'action': 'wpqd_dismiss_admin_error_filesystem'
            }
        });
    });

    // Dismiss image generation success message on "X" icon click
    $('body').on('click', '#wpqd-import-success .notice-dismiss', function (e) {
        $.ajax({
            url: ajaxurl,
            data: {
                'action': 'wpqd_dismiss_admin_import_success'
            }
        });
    });

    // Prompt imageset regeneration confirmation when clicking button
    $('body').on('click', '#wpqd_data-regenerate_images .button', function (e) {
        e.preventDefault();
        window.hifipixBatchImporter.displayConfirmation( window.hifipixBatchImporter.regenerateImages );
    });

    // Clear all image process queues
    $('body').on('click', '#wpqd_data-reset_image_queue .button', function (e) {
        e.preventDefault();
        window.hifipixBatchImporter.resetQueue();
    });

    // Pause or resume current image generation process
    $('body').on('click', '#wpqd-progress-pauser', function (e) {
        e.preventDefault();
        window.hifipixBatchImporter.pauseResumeImport();
    });

    // Detect settings changes that require imageset regeneration
    $('body').on('click', '#wpqd_data-breakpoints .redux-multi-text-remove', function () {
        window.hifipixBatchImporter.setNeedsRegenerate( true );
    });

    $('body').on('change', 'input[name="wpqd_data[breakpoints][]"], select[name="wpqd_data[crop_trim]"]', function () {
        window.hifipixBatchImporter.setNeedsRegenerate( true );
    });

    // Trigger dialog box prompt, if imageset regeneration is required
    $('body').on('click', '#wpqd-save-overlay', function (e) {
        if ( ! window.hifipixBatchImporter.needsRegenerate ) {
            $('.wpqd-settings input[name="redux_save"]').trigger('click');
            return;
        }

        e.preventDefault();
        var callback = function() {
            $('.wpqd-settings input[name="redux_save"]').trigger('click');
            window.hifipixBatchImporter.setNeedsRegenerate( false );
        };

        window.hifipixBatchImporter.displayConfirmation( callback );
    });

    // Add or remove overlay div which triggers dialog box prompt
    window.hifipixBatchImporter.setNeedsRegenerate = function ( regenerate ) {
        if ( regenerate === true ) {
            this.needsRegenerate = true;
            $('.wpqd-settings .redux-action_bar').append( '<div id="wpqd-save-overlay"></div>' );
        } else {
            this.needsRegenerate = false;
            $('#wpqd-save-overlay').remove();
        }
    };

    // Show imageset regeneration confirmation dialog box
    window.hifipixBatchImporter.displayConfirmation = function( yesCallback ) {
        new $.Zebra_Dialog('Your settings have changed and by continuing we will initiate regeneration of your imagesets. Would you like to continue?', {
            'type':     'warning',
            'title':    'Regenerate Imagesets',
            'buttons':  [
                            {caption: 'Yes', callback: function() { yesCallback() }},
                            {caption: 'No', callback: function() {}} // Do nothing
                        ]
        });
    };

    // Force regenerate all images in Tools > Regenerate Images (settings page)
    window.hifipixBatchImporter.regenerateImages = function() {
        var $fieldset = $('#wpqd_data-regenerate_images'),
            $responseArea = $('<div id="regenerate_images-response"></div>');

        $fieldset.append( '<div class="wpqd-spinner"></div>' ); // AJAX loader

        $.ajax({
            url: ajaxurl,
            data: {
                'action': 'wpqd_reimport_all_ajax'
            },
            success: function(data) {
                $('#regenerate_images-response').remove(); // Clear existing feedback
                $fieldset.append($responseArea);
                $('.wpqd-spinner', $fieldset).remove(); // Remove AJAX loader

                var $responseSelector = $('#regenerate_images-response'),
                    result = JSON.parse(data);

                if ( result ) { // Images were regenerated
                    $responseSelector
                        .addClass( 'notice notice-success' )
                        .html( '<p><strong>' + result.images + ' WPQD-enabled images were scheduled for regeneration.</strong></p>' );
                    window.location.reload(true);
                    return;
                } else {
                    $responseSelector
                        .addClass( 'notice notice-warning' )
                        .html( '<p><strong>There were no WPQD-enabled images to regenerate</strong></p>' );
                }
            }
        });
    };

    // Check image import progress and display status in admin notice bar
    window.hifipixBatchImporter.checkProgress = function() {
        $.ajax({
            url: ajaxurl,
            data: {
                'action': 'wpqd_check_progress'
            },
            success: function(data) {
                var result = JSON.parse(data),
                    $container = $('#wpqd-import-success'),
                    $button = $('#wpqd-progress-pauser', $container),
                    $message = $('#wpqd-progress-message'),
                    $currentProgress = $('.current', $message),
                    $totalCount = $('.total', $message),
                    $failedCount = $('.failed', $message),
                    $failedWrapper = $('.failed-wrapper', $message),
                    $spinner = $('.wpqd-spinner-inline', $message);

                $currentProgress.html(result.current);
                $totalCount.html(result.total);

                if ( result.failed > 0 ) {
                    $failedCount.html(result.failed);
                    $failedWrapper.show();
                }
                
                if ( result.hasOwnProperty('error') ) {
                    $container.addClass('notice-error').removeClass('notice-warning');
                    $message.append('<p>' + result.error + '</p>');
                    $spinner.hide();
                    $button.parent().hide();
                    return;
                }

                if ( result.hasOwnProperty('paused') ) {
                    if ( ! result.paused ) {
                        $spinner.show();
                    } else {
                        $spinner.hide();
                        return;
                    }
                }

                if ( ! result.hasOwnProperty('complete') ) {
                    // Keep checking the progress until the import job is finished
                    setTimeout( 
                        function() { window.hifipixBatchImporter.checkProgress() },
                        3000
                    );
                } else {
                    $container.addClass('notice-success').removeClass('notice-warning');
                    $spinner.hide();
                    $button.parent().hide();
                }
            }
        });
    };

    // Pause or resume the current image process, update admin notice bar
    window.hifipixBatchImporter.pauseResumeImport = function() {
        var $container = $('#wpqd-import-success'),
            $button = $('#wpqd-progress-pauser', $container);

        $container.append( '<div class="wpqd-spinner"></div>' ); // AJAX loader

        $.ajax({
            url: ajaxurl,
            data: {
                'action': 'wpqd_pause_resume_import'
            },
            success: function(data) {
                var result = JSON.parse(data);
                $('.wpqd-spinner', $container).remove();
                
                if ( ! result.hasOwnProperty('paused') ) {
                    return;
                }

                if ( ! result.paused ) {
                    $button.html( 'Pause Import' );
                    window.hifipixBatchImporter.checkProgress();
                } else {
                    $button.html( 'Resume Import' );
                }
            }
        });
    };

    // Clear all currently scheduled image process queues
    window.hifipixBatchImporter.resetQueue = function() {
        var $fieldset = $('#wpqd_data-reset_image_queue'),
            $responseArea = $('<div id="reset_image_queue-response"></div>'),
            $progressDismiss = $('#wpqd-import-success .notice-dismiss');

        $fieldset.append( '<div class="wpqd-spinner"></div>' ); // AJAX loader

        if ( $progressDismiss.length > 0 ) {
            $progressDismiss.trigger( 'click' );
        }

        $.ajax({
            url: ajaxurl,
            data: {
                'action': 'wpqd_reset_queue'
            },
            success: function(data) {
                $( '#reset_image_queue-response' ).remove(); // Clear existing feedback
                $fieldset.append( $responseArea );
                $( '.wpqd-spinner', $fieldset ).remove(); // Remove AJAX loader

                var $responseSelector = $('#reset_image_queue-response'),
                    result = JSON.parse(data);

                if ( result > 0 ) { // Queues were cleared
                    $responseSelector
                        .addClass( 'notice notice-success' )
                        .html( '<p><strong>The image process queue has been reset.</strong></p>' );
                    return;
                } else {
                    $responseSelector
                        .addClass( 'notice notice-warning' )
                        .html( '<p><strong>Nothing happened because the image process queue was empty.</strong></p>' );
                }
            }
        });
    };
});
