<?php

/* ------------------------------------------------------------------------ */
/* Redux Configuration
/* ------------------------------------------------------------------------ */
if ( ! class_exists( 'Redux' ) ) {
    return;
}

// This is your option name where all the Redux data is stored.
global $wpqd_opt_name;
$wpqd_opt_name = WPQD_OPTIONS;

$args = array(
    'opt_name'          => $wpqd_opt_name,          // This is where your data is stored in the database and also becomes your global variable name.
    'display_name'      => WPQD_NAME,               // Name that appears at the top of your panel
    'display_version'   => WPQD_VERSION,            // Version that appears at the top of your panel
    'menu_type'         => 'menu',                  // Specify if the admin menu should appear or not. Options: menu or submenu (Under appearance only)
    'allow_sub_menu'    => false,                   // Show the sections below the admin menu item or not
    'menu_title'        => __(WPQD_NAME, 'wpqd'),
    'page_title'        => __(WPQD_NAME, 'wpqd'),
    'save_defaults'     => true,
    'async_typography'  => true,                     // Use a asynchronous font on the front end or font string
    'admin_bar'         => false,                    // Show the panel pages on the admin bar
    'dev_mode'          => false,                    // Show the time the page took to load, etc
    'customizer'        => false,                    // Enable basic customizer support
    'page_slug'         => 'wpqd_options_page',
    'system_info'       => false,
    'disable_save_warn' => true,                    // Disable the save warning when a user changes a field
    'menu_icon'         => 'none',
    'class'             => 'wpqd-settings',
    'show_import_export' => false,
    'global_variable'   => $wpqd_opt_name,
    'footer_credit'		=> '&nbsp;'
);

Redux::setArgs( $wpqd_opt_name, $args );

/**
 * Behavior
 */
Redux::setSection( $wpqd_opt_name, array(
    'title'     => __('Behavior', 'wpqd'),
    'heading'   => __('General Settings', 'wpqd'),
    'icon'      => 'el el-screen',
    'fields'    => array(
        array(
            'id'       => 'enabled',
            'type'     => 'switch',
            'title'    => __('Enable Plugin', 'wpqd'),
            'subtitle' => __('Toggle QuickDraw on/off sitewide<br><a href="https://www.wpquickdraw.com/documentation/settings/#enableplugin" class="wpqd_documentation" target="_blank">Learn More</a>', 'wpqd'),
            'default'  => true,
        ),

        array(
            'id'        => 'file_types',
            'type'      => 'checkbox',
            'title'     => __('Apply To File Types', 'wpqd'),
            'subtitle'  => __('Check the image types you would like to QuickDraw', 'wpqd'),
            'options'   => array(
                'png' => 'PNG',
                'jpg' => 'JPEG',
                'gif' => 'GIF'
            ),
            'default'   => array(
                'png' => '1',
                'jpg' => '1',
                'gif' => '1'
            ),
            'class' => 'hidden'
        ),

        array(
            'id'    => 'behavior-header',
            'type'  => 'info',
            'title' => __('Behavior', 'wpqd')
        ),

        array(
            'id'        => 'enhancement_behavior',
            'type'      => 'slider',
            'title'     => __('Image Enhancement Behavior', 'wpqd'),
            'subtitle'  => __('Select an image enhancement behavior: Aggressive, Optimized (recommended), or Lazy.<br><a href="https://www.wpquickdraw.com/documentation/settings/#enhancement" class="wpqd_documentation" target="_blank">Learn More</a>', 'wpqd'),
            'desc'      => '<ul class="behavior-labels"><li id="behavior-label-1">Lazy</li><li id="behavior-label-2">Optimized</li><li id="behavior-label-3">Aggressive</li></ul>
            <div id="wpqd_custom_behavior">Custom Behavior Settings Enabled</div>',
            'default'   => 2,
            'min'       => 1,
            'step'      => 1,
            'max'       => 3,
            'display_value' => 'label'
        ),

        array(
            'id'       => 'enhancement_behavior_custom',
            'type'     => 'switch',
            'title'    => __('Custom Behavior Settings Enabled', 'wpqd'),
            'default'  => false,
            'class'    => 'hidden'
        ),

        array(
            'id'       => 'progress',
            'type'     => 'switch',
            'title'    => __('Progress Indicator', 'wpqd'),
            'subtitle' => __('Toggle this setting OFF to hide the progress indicator as images enhance<br><a href="https://www.wpquickdraw.com/documentation/settings/#progressindicator" class="wpqd_documentation" target="_blank">Learn More</a>', 'wpqd'),
            'default'  => false,
        ),
    ),
) );

/**
 * Advanced Configuration
 */
Redux::setSection( $wpqd_opt_name, array(
    'title'     => __('Advanced Configuration', 'wpqd'),
    'icon'      => 'el el-cogs',
    'id'        => 'advanced-config',
    'class'     => 'advanced-config',
    'fields'    => array(

        array(
            'id'       => 'deferred_load',
            'type'     => 'switch',
            'title'    => __('Deferred Load', 'wpqd'),
            'subtitle' => __('<a href="https://www.wpquickdraw.com/documentation/settings/#deferredload" class="wpqd_documentation" target="_blank">Learn More</a>'),
            'default'  => true,
        ),

        array(
            'id'       => 'trigger',
            'type'     => 'select',
            'title'    => __('Trigger Type', 'wpqd'),
            'subtitle' => __('<a href="https://www.wpquickdraw.com/documentation/settings/#triggertype" class="wpqd_documentation" target="_blank">Learn More</a>'),
            'options'  => array(
                'scroll' => 'Scroll',
                'hover' => 'Hover'
            ),
            'default'  => 'scroll',
        ),

        array(
            'id'             => 'scroll_sensitivity',
            'type'           => 'text',
            'validate_callback' => 'wpqd_validate_scroll_sensitivity',
            'title'          => __('Scroll Load Sensitivity', 'wpqd'),
            'subtitle'       => __(
                'Defines the threshold from the bottom of the viewport to the top of image when image enhancement starts during scroll. Enter a decimal value between 0 and 1.<br><a href="https://www.wpquickdraw.com/documentation/settings/#scrollload" class="wpqd_documentation" target="_blank">Learn More</a>', 
                'wpqd'
            ),
            'default'        => '0.25'
        ),

        array(
            'id'       => 'placeholder',
            'type'     => 'select',
            'title'    => __('Initial Image State', 'wpqd'),
            'subtitle' => __('<a href="https://www.wpquickdraw.com/documentation/settings/#initialimage" class="wpqd_documentation" target="_blank">Learn More</a>'),
            'options'  => array(
                'low' => 'Low Resolution Image',
                'placeholder' => 'Placeholder Image',
                'transparent' => 'Transparent (Spacer GIF)'
            ),
            'default'  => 'low',
        ),

        array(
            'id'       => 'placeholder_img',
            'type'     => 'media',
            'url'      => false,
            'title'    => __('Placeholder Image', 'wpqd'),
        ),

        array(
            'id'       => 'js_support_msg',
            'type'     => 'switch',
            'title'    => __('Javascript Support Load Message', 'wpqd'),
            'subtitle' => __('<a href="https://www.wpquickdraw.com/documentation/settings/#javascriptsupport" class="wpqd_documentation" target="_blank">Learn More</a>'),
            'default'  => true,
        ),

        array(
            'id'       => 'crop_trim',
            'type'     => 'select',
            'title'    => __('Crop/Trim Settings', 'wpqd'),
            'options'  => array(
                'trim' => 'Trim',
                'trans_pixels' => 'Add Transparent Pixels',
            ),
            'default'  => 'trim',
            'class' => 'hidden'
        ),

        array(
            'id'       => 'seo_user_agent',
            'type'     => 'textarea',
            'title'    => __('Custom SEO Bot User Agent Strings', 'wpqd'),
            'subtitle' => __('One value per line (default value "Industry Standard bots" will include most search engine crawlers)<br><a href="https://www.wpquickdraw.com/documentation/settings/#customseo" class="wpqd_documentation" target="_blank">Learn More</a>', 'wpqd'),
            'default'  => 'Industry Standard bots',
        ),

        array(
            'id'       => 'breakpoints',
            'type'     => 'multi_text',
            'title'    => __('Web Responsive Breakpoints', 'wpqd'),
            'subtitle' => __('Screen width in pixels<br><a href="https://www.wpquickdraw.com/documentation/settings/#breakpoints" class="wpqd_documentation" target="_blank">Learn More</a>', 'wpqd'),
            'validate_callback' => 'wpqd_validate_breakpoints',
            'default'  => array(
                '768',
                '1024',
                '1200',
                '2000'
            )
        ),

    ),
) );
Redux::hideSection( $wpqd_opt_name, 'advanced-config', true );

/**
 * Tools
 */
Redux::setSection( $wpqd_opt_name, array(
    'title'     => __('Tools', 'wpqd'),
    'icon'      => 'el el-wrench',
    'fields'    => array(

        array(
            'id'       => 'compatibility_check',
            'type'     => 'raw',
            'title'    => __('Compatibility Check', 'wpqd'),
            'subtitle' => __('Perform compatibility check on this WordPress site.<br><a href="https://www.wpquickdraw.com/documentation/settings/#compatibilitycheck" class="wpqd_documentation" target="_blank">Learn More</a>', 'wpqd'),
            'content'  => '<div class="button button-primary">Perform Check</div>',
            'full_width' => false
        ),

        array(
            'id'       => 'regenerate_images',
            'type'     => 'raw',
            'title'    => __('Regenerate Images', 'wpqd'),
            'subtitle' => __('Regenerate image sets using current plugin settings.<br><a href="https://www.wpquickdraw.com/documentation/settings/#regenerateimages" target="_blank">Learn More</a>', 'wpqd'),
            'content'  => '<div class="button button-primary">Regenerate Images</div>',
            'full_width' => false
        ),

        array(
            'id'       => 'reset_image_queue',
            'type'     => 'raw',
            'title'    => __('Reset Image Process Queue', 'wpqd'),
            'subtitle' => __('Clear the image set process queue (this will cancel any imports currently in progress).<br><a href="https://www.wpquickdraw.com/documentation/settings/#resetqueue" class="wpqd_documentation" target="_blank">Learn More</a>', 'wpqd'),
            'content'  => '<div class="button button-primary">Reset Queue</div>',
            'full_width' => false
        ),
        
        array(
            'id'       => 'image_status_page',
            'type'     => 'raw',
            'title'    => __('View Image Status', 'wpqd'),
            'subtitle' => __('See a detailed breakdown of images & if they have been processed by WP QuickDraw.<br><a href="https://www.wpquickdraw.com/documentation/settings/#viewimagestatus" class="wpqd_documentation" target="_blank">Learn More</a>', 'wpqd'),
            'content'  => sprintf(
                '<a href="%s"><div class="button button-primary">View Image Status Page</div></a>',
                get_site_url( null, '/wp-admin/tools.php?page=wpqd_image_status_page' )
            ),
            'full_width' => false
        ),

    ),
) );

/**
 * Custom validation callback for "Web Responsive Breakpoints"
 * 
 * @param   array   $field
 * @param   array   $value
 * @param   array   $existing_value
 * @return  array
 */
function wpqd_validate_breakpoints( $field, $value, $existing_value ) {
    $return = array(
        'value' => $existing_value
    );

    if ( count($value) > 6 ) {
        $field['msg'] = __('You may only define a maximum of 6 breakpoints.', 'wpqd');
        $return['error'] = $field;
        return $return;
    }

    foreach ( $value as $breakpoint ) {
        if ( is_numeric($breakpoint) && is_int($breakpoint + 0) ) {
            continue;
        }

        $field['msg'] = sprintf(
            __('You may only define a valid integer as a breakpoint. "%s" is not an integer.', 'wpqd'),
            esc_html( $breakpoint )
        );
        $return['error'] = $field;
        return $return;
    }

    $return['value'] = $value;
    return $return;
}

/**
 * Custom validation callback for "Scroll Sensitivity"
 * 
 * @param   array   $field
 * @param   array   $value
 * @param   array   $existing_value
 * @return  array
 */
function wpqd_validate_scroll_sensitivity( $field, $value, $existing_value ) {
    $return = array(
        'value' => $existing_value
    );

    if  ( ! is_numeric($value) || ! is_float($value + 0) ) {
        $field['msg'] = __('You must enter a decimal value.', 'wpqd');
        $return['error'] = $field;
        return $return;
    }

    if ( $value > 1 || $value < 0 ) {
        $field['msg'] = __('You must enter a value between 0 and 1.', 'wpqd');
        $return['error'] = $field;
        return $return;
    }

    $return['value'] = $value;
    return $return;
}

/**
 * Apply bold class to current "Image Enhancement Behavior" field label
 *
 * @param   array   $field
 * @return  array
 */
function wpqd_enhancement_behavior_init( $field ) {
    if ( !isset($field['desc']) || strpos($field['desc'], 'behavior-label-') === false ) {
        return;
    }

    $options = get_option(WPQD_OPTIONS);
    $current_value = isset( $field['default'] ) ? $field['default'] : false;

    if ( isset($options[$field['id']]) ) {
        $current_value = $options[$field['id']];
    }

    if ( !$current_value ) {
        return;
    }

    $html_id = 'id="behavior-label-' . $current_value . '"';

    $field['desc'] = str_replace(
        $html_id,
        $html_id . ' class="strong"',
        $field['desc']
    );

    return $field;
}
add_action( 'redux/options/' . $wpqd_opt_name . '/field/enhancement_behavior', 'wpqd_enhancement_behavior_init' );

/**
 * Save settings as JSON-encoded string in {uploads_dir}/wpqd/settings.js
 *
 * @param   array   $this_options
 * @return  void
 * @throws  Exception
 */
function wpqd_save_config_file($this_options) {
    WP_Filesystem();
    global $wp_filesystem;

    $uploads = wp_upload_dir();
    $path = trailingslashit($uploads['basedir'] . WPQD_UPLOADS_FOLDER);
    $file = $path . 'settings.js';

    try {
        if ( ! $wp_filesystem->is_writable($uploads['basedir']) ) {
            throw new Exception( sprintf('Your uploads directory must be writable: %1$s', $uploads['basedir']) );
        }

        if ( ! $wp_filesystem->is_dir($path) ) {
            if ( ! $wp_filesystem->mkdir($path) ) {
                throw new Exception( sprintf('Unable to create directory: %1$s', $path) );
            }
        }

        if ( ! $wp_filesystem->put_contents( $file, wpqd_format_options($this_options) ) ) {
            throw new Exception( sprintf('Unable to save settings to file: %1$s', $file) );
        }
    } catch ( Exception $e ) {
        $messages = array(
            $e->getMessage(),
            sprintf( 'WordPress is using this class to interact with your filesystem: %1$s', get_class($wp_filesystem) )
        );
        set_transient( PluginState::TRANSIENT_KEY_FILESYSTEM, $messages );
    }
}
add_action ('redux/options/' . $wpqd_opt_name . '/saved', 'wpqd_save_config_file');

/**
 * Format settings for save to {uploads_dir}/wpqd/settings.js
 *
 * @param   array   $options
 * @return  string
 */
function wpqd_format_options( $options ) {
    global $wpqd_opt_name;
    $formatted_options = array();
    $include_sections = array( 'behavior', 'advanced-config' );
    $skip_fields = array( 'enhancement_behavior', 'advanced_config' );
    $general_fields = array( 'enabled', 'file_types' );

    foreach ( Redux::constructSections($wpqd_opt_name) as $section ) {
        if ( !isset($section['id']) || !in_array( $section['id'], $include_sections ) ) {
            continue;
        }

        if ( !isset($section['fields']) || empty($section['fields']) ) {
            continue;
        }

        // Put advanced configuration options in "Behavior" section
        $section_id = ($section['id'] === 'advanced-config') ? 'behavior' : $section['id'];

        foreach ( $section['fields'] as $field ) {
            if ( !isset($field['id']) || !isset($options[$field['id']]) ) {
                continue;
            }

            if ( in_array( $field['id'], $skip_fields ) ) {
                continue;
            }

            $current_section_id = $section_id;
            if ( in_array( $field['id'], $general_fields ) ) {
                $current_section_id = 'general';
            }

            $formatted_options[$current_section_id][$field['id']] = $options[$field['id']];
        }
    }

    // TODO: Make these not hard-coded, but based on a particular image set
    $formatted_options['behavior']['ratios'] = array('1_8', '1_4', '1_2', '1_1');

    return 'window.hifipix_settings = ' . json_encode($formatted_options, JSON_PRETTY_PRINT);
}

/**
 * Reverse position of "On" and "Off" labels in toggle switch field
 * 
 * @param   string  $render
 * @param   array   $field
 * @return  string
 */
function wpqd_field_switch_output( $render, $field ) {
    $dom = new DOMDocument();
    $dom->loadHTML( $render );
    $target = $parent = $disabled = false;

    foreach ( $dom->getElementsByTagName('label') as $label ) {
        $classes = $label->getAttribute('class');

        if ( strpos($classes, 'cb-enable') !== false ) {
            $target = $label;
            $parent = $label->parentNode;
        } elseif ( strpos($classes, 'cb-disable') !== false ) {
            $disabled = $label;
        }

        if ( $target && $disabled ) {
            break;
        }
    }

    if ( ! $target || ! $parent || ! $disabled ) {
        return $render;
    }

    // Swap positions of labels
    $parent->removeChild( $disabled );
    $parent->insertBefore( $disabled, $target );

    return $dom->saveHTML( $parent );
}
add_action( 'redux/field/' . $wpqd_opt_name . '/switch/render/after', 'wpqd_field_switch_output', 10, 2 );

/**
 * Display pro version ad at top of settings page
 * 
 * @return  void
 */
function wpqd_pro_advertisement() {
    ?>
    <div id="wpqd_admin_advertisement" class="wpqd_license_advertisement <?php do_action( 'wpqd/redux/advertisement' ); ?>">
        <h1>Get WP QuickDraw PRO for next generation features</h1>
        <img src="https://assets.hifipix.com/wp-content/uploads/2018/07/rocket-1.png" />
        <span>
            WP QuickDraw can be upgraded with a WP QuickDraw PRO annual subscription to add “on-demand” resolution whenever your users zoom in on your images, realistic fade-in technology for the most elegant image presentation, and our new progressive PNG technology for even faster delivery of lossless images (coming soon). You also get advanced QuickDraw settings, free PRO updates, plugin technical support from HifiPix and community support and discussions through the QuickDraw forum. All today's and tomorrow's features and support for just <strong>$49.95 per year</strong>.
        </span>
        <br />
        <a href="https://www.wpquickdraw.com/pro/" target="_blank">Upgrade to PRO</a>
    </div>
    <?php
}
add_action( 'redux/page/' . $wpqd_opt_name . '/form/before', 'wpqd_pro_advertisement' );

/**
 * Add "pro disabled" class to advertisement wrapper
 * 
 * @return  void
 */
function wpqd_advertisement_class() {
    echo 'pro-disabled';
}
add_action( 'wpqd/redux/advertisement', 'wpqd_advertisement_class' );

/**
 * Add link to settings page from Plugins list
 *
 * @param   array   $links
 * @return  array
 */
function wpqd_plugin_settings_link( $links ) {
    $links[] = '<a href="admin.php?page=wpqd_options_page">' . __( 'Settings', 'wpqd' ) . '</a>';
  	return $links;
}
add_filter( 'plugin_action_links_' . WPQD_BASENAME, 'wpqd_plugin_settings_link' );

/**
 * Custom CSS stylesheets & JS scripts for admin settings page
 *
 * @return  void
 */
function wpqd_admin_enqueue_scripts() {
    // Google fonts
    wp_enqueue_style( 'wpqd-fonts-raleway', '//fonts.googleapis.com/css?family=Raleway:300,400,500,600,700,800', array(), WPQD_VERSION );
    
    // Zebra Dialog - @see https://github.com/stefangabos/Zebra_Dialog
    wp_enqueue_style( 'wpqd-zebra-dialog-styles', WPQD_URL . 'assets/css/zebra_dialog/zebra_dialog.min.css', array(), WPQD_VERSION );
    wp_enqueue_script('wpqd-zebra-dialog-script', WPQD_URL . 'assets/js/zebra_dialog.min.js', array( 'jquery' ), WPQD_VERSION);

    // Custom styles & scripts
    wp_enqueue_style( 'wpqd-admin-styles', WPQD_URL . 'assets/css/admin.css', array( 'wpqd-fonts-raleway', 'wpqd-zebra-dialog-styles' ), WPQD_VERSION );
    wp_enqueue_script('wpqd-admin-script', WPQD_URL . 'assets/js/admin.js', array( 'jquery', 'redux-nouislider-js', 'redux-js', 'redux-field-slider-js' ), WPQD_VERSION);
    wp_enqueue_script('wpqd-admin-ml-script', WPQD_URL . 'assets/js/media-library.js', array( 'jquery' ), WPQD_VERSION);
    wp_enqueue_script('wpqd-admin-plugin-state', WPQD_URL . 'assets/js/plugin-state.js', array( 'jquery', 'wpqd-zebra-dialog-script' ), WPQD_VERSION);
}
add_action( 'admin_enqueue_scripts', 'wpqd_admin_enqueue_scripts' );

do_action( 'wpqd/redux/init', $wpqd_opt_name );
