<?php
/**
 * WP QuickDraw is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * WP QuickDraw is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with WP QuickDraw. If not, see http://www.gnu.org/licenses/gpl.html.
 * 
 * @package     WP_QuickDraw
 * @author      hifipix
 * @copyright   2018 HifiPix, Inc.
 * @license     http://www.gnu.org/licenses/gpl.html  GNU GPL, Version 3
 * @version     1.0.1
 */
 
namespace Wpqd;

use \WP_Image_Editor_Imagick;

class ImageEditor {

  private $path;
  private $imageEditor;

  public function __construct($path) {
    $this->path = $path;

    // Get GD or Imagick version based on compatibility checks
    // TODO: integrate with global compat check
    if (extension_loaded('gd')){
      // error_log('GD IMAGES');
      $this->imageEditor = wp_get_image_editor($this->path);  
      // error_log(var_export($this->imageEditor, true));
    } elseif (extension_loaded('imagick')) {
      // error_log('IMAGICK IMAGES');
      $this->imageEditor = new WP_Image_Editor_Imagick($this->path);
    }
  }

  public function resize($sizes) {
    // TODO: Transform $sizes using intended resize logic
    // and to make the structure of input $sizes more simple
    // error_log('SIZE');
    // error_log(var_export($sizes, true));

    // Generate sizes and save to disk
    // error_log('METADATA');
    $resizedFiles = $this->imageEditor->multi_resize($sizes);
    // error_log(var_export($resizedFiles, true));

    return $resizedFiles;
  }

  /**
   * Test one resize to check for WP_Error
   * 
   * @param $size array
   * @return  string|bool
   */
  public function testResize( $size ) {
    $resized = $this->imageEditor->resize( $size['width'], $size['height'], $size['crop'] );

    if ( is_wp_error( $resized ) ) {
      return $resized->get_error_message();
    }
    return false;
  }

  /**
   * Check if $imageEditor was instantiated successfully
   * 
   * @return  string|bool
   */
  public function getErrorMessage() {
    if ( is_wp_error( $this->imageEditor ) ) {
      return $this->imageEditor->get_error_message();
    }
    return false;
  }

  public function getSize() {
    return $this->imageEditor->get_size();
  }

  public function crop() {
    
  }

  // public function __clone() {
  //     $original = $this->_img;
  //     $copy = imagecreatetruecolor($this->_width, $this->_height);

  //     imagecopy($copy, $original, 0, 0, 0, 0, $this->_width, $this->_height);

  //     $this->_img = $copy;
  // }

}
